use axum::http::StatusCode;
use axum::{
    routing::{get, post},
    Json, Router,
};
use serde_json::Value;

use crate::event_type::get_event_type_map;
use typing::EventMessage;

mod event_type;
mod typing;

async fn event_rcv_handle(
    Json(form_data): Json<EventMessage>,
) -> Result<String, (StatusCode, String)> {
    tokio::spawn(async move {
        println!("收到告警，向飞书推送消息");
        let event_map = get_event_type_map();
        for event in &form_data.params.events {
            let event_type = match event_map.get(&event.event_type) {
                None => String::from("未知事件类型"),
                Some(name) => name.to_string(),
            };
            let _ = reqwest::get(format!("https://www.feishu.cn/flow/api/trigger-webhook/31f13dead0bf78fc4bdb51ba23abba9f?title={}&content=发生时间：{}", event_type, event.happen_time)).await;
        }
    });
    Ok(String::from("success"))
}

async fn event_rcv_handle_v2(
    Json(form_data): Json<Value>,
) -> Result<String, (StatusCode, String)> {
    println!("{}", form_data.to_string());
    tokio::spawn(async move {
        println!("收到告警，向飞书推送消息");
        let _ = reqwest::get(format!("https://www.feishu.cn/flow/api/trigger-webhook/31f13dead0bf78fc4bdb51ba23abba9f?title={}&content={}", "收到告警", "告警内容")).await;
    });
    Ok(String::from("success"))
}

#[tokio::main]
async fn main() {
    // build our application with a single route
    let app = Router::new()
        .route("/", get(|| async { "Hello, World!" }))
        .route("/eventRcv", post(event_rcv_handle))
        .route("/eventRcv2", post(event_rcv_handle_v2));

    // run our app with hyper, listening globally on port 3000
    let listener = tokio::net::TcpListener::bind("0.0.0.0:13000")
        .await
        .unwrap();
    axum::serve(listener, app).await.unwrap();
}
